@@ -132,8 +132,6 @@ struct _GrdRdpPipeWireStream
   uint32_t src_node_id;
 
   struct spa_video_info_raw spa_format;
-
-  gboolean destroyed;
 };
 
 G_DEFINE_TYPE (GrdRdpPipeWireStream, grd_rdp_pipewire_stream,
@@ -758,7 +756,7 @@ process_frame_data (GrdRdpPipeWireStream *stream,
       int *fds;
       uint32_t *offsets;
       uint32_t *strides;
-      uint64_t *modifiers;
+      uint64_t *modifiers = NULL;
       uint32_t n_planes;
       unsigned int i;
       uint8_t *dst_data = NULL;
@@ -779,14 +777,16 @@ process_frame_data (GrdRdpPipeWireStream *stream,
       fds = g_alloca (sizeof (int) * n_planes);
       offsets = g_alloca (sizeof (uint32_t) * n_planes);
       strides = g_alloca (sizeof (uint32_t) * n_planes);
-      modifiers = g_alloca (sizeof (uint64_t) * n_planes);
+      if (stream->spa_format.modifier != DRM_FORMAT_MOD_INVALID)
+        modifiers = g_alloca (sizeof (uint64_t) * n_planes);
 
       for (i = 0; i < n_planes; i++)
         {
           fds[i] = buffer->datas[i].fd;
           offsets[i] = buffer->datas[i].chunk->offset;
           strides[i] = buffer->datas[i].chunk->stride;
-          modifiers[i] = stream->spa_format.modifier;
+          if (modifiers)
+            modifiers[i] = stream->spa_format.modifier;
         }
 
       if (!stream->rdp_surface->needs_no_local_data)
@@ -976,9 +976,6 @@ on_stream_process (void *user_data)
   struct pw_buffer *last_frame_buffer = NULL;
   struct pw_buffer *next_buffer;
 
-  if (stream->destroyed)
-    return;
-
   while ((next_buffer = pw_stream_dequeue_buffer (stream->pipewire_stream)))
     {
       if (grd_pipewire_buffer_has_pointer_bitmap (next_buffer))
@@ -1287,21 +1284,15 @@ grd_rdp_pipewire_stream_finalize (GObject *object)
   GrdContext *context = grd_session_get_context (session);
   GrdEglThread *egl_thread;
 
-  stream->destroyed = TRUE;
-
+  /* Setting a PipeWire stream inactive will wait for the data thread to end */
   if (stream->pipewire_stream)
-    pw_stream_flush (stream->pipewire_stream, false);
+    pw_stream_set_active (stream->pipewire_stream, false);
 
   egl_thread = grd_context_get_egl_thread (context);
   if (egl_thread && stream->pipewire_stream)
     sync_egl_thread (egl_thread);
 
-  /*
-   * We can't clear stream->pipewire_stream before destroying it, as the data
-   * thread in PipeWire might access the variable during destruction.
-   */
-  if (stream->pipewire_stream)
-    pw_stream_destroy (stream->pipewire_stream);
+  g_clear_pointer (&stream->pipewire_stream, pw_stream_destroy);
 
   g_clear_pointer (&stream->pipewire_core, pw_core_disconnect);
   g_clear_pointer (&stream->pipewire_context, pw_context_destroy);
